c+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c
c       *******************************************************************
c       ** THIS FORTRAN CODE IS INTENDED TO ILLUSTRATE POINTS MADE IN    **
c       ** THE CORRESPONDING PAPER PUBLISHED ON COMPUTATIONAL MATERIALS  **
c       ** SCIENCE.                                                      **
c       ** TO THE BEST OF OUR KNOWLEDGE IT WORKS CORRECTLY.              **
c       ** HOWEVER IT IS THE RESPONSIBILITY OF THE USER TO TEST IT,      **
c       ** IF IT IS USED IN A RESEARCH APPLICATION.                      ** 
c       *******************************************************************
c
c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c
c
c
c****************************************************************************
c****************************************************************************
      program TBMD
c****************************************************************************
c****************************************************************************
c
c     RELEASE TBMD.2.0
c
c****************************************************************************
c****************************************************************************
c
c     Date of creation:  15/03/96 
c     Authors:           Giorgio Servalli and Luciano Colombo
c                        Departement of Materials Science - University of Milano
c                        via Emanueli 15 
c                        I-20126 Milano
c                        Italy
c     Contact author:    Luciano Colombo
c                        tel. +39 2 66174218 
c                        fax  +39 2 66174403 or 66174400 
c                        e-mail:   luciano.colombo@mater.unimi.it
c
c****************************************************************************
c****************************************************************************
c
c     OUTLINE
c     -------
c     This is the updated version of the code TBMD (distributed on
c     October 24, 1994 as release #1).
c     The interaction hamiltonian has been changed from the model by
c     Goodwin et al. (Europhys. Lett. 9, 701 (1989) to the new scheme 
c     described in:
c     - C.H.Xu, C.Z. Wang, C.T. Chan, and K.M. Ho, J.Phys.Condens.Matter, vol.4, 
c       pg.6047 (1992) 
c     - I. Kwon, R. Biswas, C.Z. Wang, K.M. Ho, and C.M. Soukolis, Phys. 
c       Rev. B49, pg.7242 (1994)
c
c****************************************************************************
c****************************************************************************
c
c     OUTLINE
c     -------
c     This code is designed to run tight-binding molecular dynamics (TBMD)
c     simulations in both canonical and microcanonical ensambles.
c     The present release of the TBMD code works for silicon-based and carbon-based
c     systems. Three-dimensional periodicity is assumed. The density is 
c     arbitrary (see below).
c 
c     A good introduction to molecular dynamics is described in the book:
c        M.P. Allen and D.J. Tildesley, ``Computer simulations of liquids''
c        (Oxford Science Publications, New York, 1990)
c     The present TBMD computational framework is described in:
c       i.) tight-binding formalism:
c           J.C. Slater and G.F. Koster, Phys. Rev. vol.94, pg.1498 (1954)
c      ii.) TBMD scheme:
c           - L. Colombo, Annual Reviews of Computational Physics, volume IV, 
c             Edited by D. Stauffer 
c             (World Scientific Publishing Co., Singapore, 1996) 
c   
c     A number of physical observables can be computed during a typical TBMD
c     run. They are:
c     STRUCTURAL PROPERTIES: 		1. particle-particle correlation function
c					2. radial distribution function
c					3. bond-angle distribution function
c					4. atomic coordination number
c                                       5. bond-length distribution function
c     ELECTRONIC PROPERTIES:		6. total electronic density of states
c     DYNAMICAL PROPERTIES:		7. atomic mean square displacement 
c     THERMODYNAMICAL PROPERIES:	8. temperature
c					
c     Moreover, atomic trajectories and istantaneous velocities can be saved and
c     stored for post-processing.
c
c     A more detailed description of the basic formalism and program organization
c     can be found elsewhere (see hard-copy documentation of the present 
c     code available from PECCAM).
c     
c     REMARKS
c     -------
c     1. This is the *SECOND* release of the TBMD code. 
c     2. The source code consists in one main program, 14 subroutines, one function,
c        17 include files and one input file.
c     3. The list of subroutines is (more information is reported in any routine):
c        subroutine bondlength: compute bond-length distribution function
c        subroutine correlate : compute particle-particle correlation function and
c                               radial distributon function
c        subroutine crystal   : assigne tight-binding and repulsive potential parameters
c    	 subroutine dosenergy : compute total electronic density of states
c      	 subroutine feynman   : compute Hellmann-Feynman forces 
c      	 subroutine htb       : compute and diagonalizes the tight-binding matrix;
c                               compute band-structure energy
c      	 subroutine init_pos  : assigne initial positions 
c      	 subroutine init_vel  : assigne initial velocities
c      	 subroutine md        : molecular-dynamics loop
c      	 subroutine meandisp  : compute the atomic mean square displacement
c      	 subroutine nearangle : compute bond-angle distribution function and
c                               atomic coordination number
c      	 subroutine repuls    : compute repulsive energy and forces
c      	 subroutine temper    : compute temperature
c     4. The function name is RAN1: it generates a random number in the interval ]0,1[
c     5. Main program, subroutines and the function are contained in the present file.
c     6. The list of include files is (more information is reported in any file):
c        acc.inc           \
c	 dist.inc           \
c	 eigen.inc          |
c	 erg.inc	    |
c	 fhf.inc	    |
c	 frep.inc	    |
c	 ftot.inc	    |
c	 func.inc	    |
c	 latt.inc	    |  they assigne the dimensions to suitable arrays 
c	 lst.inc	    |
c	 param.inc	    |
c	 pos.inc	    |
c	 potz.inc	    |
c	 simul.inc	    |
c	 start.inc	    |
c	 tb.inc	            /
c	 vel.inc	   /	
c     7. The input file is tbmd.in  
c     8. A routine to diagonalize the tight-binding hamiltonian matrix is
c        needed (see subroutine htb). In the present release of the code, the
c        LAPACK library is used to this aim.
c
c***************************************************************************
c***************************************************************************
c
c     LIST OF MAIN VARIABLES 
c     ----------------------
c
c     - itype           : select the material.
c                         1 = Carbon 
c                         2 = Silicon
c                        >2 = not implemented 
c     - ncell           : number of replicas of the conventional diamond cell 
c                         (8-atom cell) along the x,y,z directions. It defines the
c                         size of the simulation box (see subroutine crystal) 
c     - n               : number of atoms in the simulation box. It is given by:
c                         n=8*ncell**3
c     - x/y/z(i)        : x,y,z component of the position vector of the i-th atom 
c                         (reduced units: Angstrom  -- abbreviation: A)
c     - vx/y/z(i)       : x,y,z component of the velocity of the i-th atom  
c                         (reduced units: sqrt(eV/amu)= 9.822694E+13 A/s)
c     - ax/y/z(i)       : x,y,z component of the acceleration of the i-th atom
c                         (reduced units: eV/A/amu = 9.648532E+27 A/s^2)
c     - ax/y/zold (i)   : x,y,z component of the acceleration of the i-th atom 
c                         at the previous step
c                         (reduced units: eV/A/amu = 9.648532E+27 E/s^2)
c     - xx/yy/zz(i,j)   : x,y,z component of the distance vector between atoms j and i
c                         ( xx(i,j)= x(j)-x(i), yy(i,j)=...... )
c                         (reduced units: A)
c     - rr2(i,j)        : squared relative distance between atoms j and i 
c                         (reduced units: A^2)
c     - tempi,tempf     : initial and final temperature 
c                         (reduced units: Kelvin)
c     - temp            : istantaneous temperature
c                         (reduced units: Kelvin)
c     - cell            : lenght of the cubic simulation box (see subroutine crystal)
c                         (reduced units: A)
c     - eval(n4)        : eigenvalues of tight-binding hamiltonian matrix
c     - utot            : repulsive potential energy per atom (eV)
c     - ebstot          : band-structure energy per atom (eV)
c     - kintot          : kinetic energy per atom (eV)
c     - energy          : total energy per atom (eV)
c     - momx/y/z        : x,y,z component of the total linear momentum of the system 
c                         (reduced units: sqrt(eV*amu)= 1.6311E-18 g*cm/s)
c     - frepx/y/z(i)    : x,y,z component of the repulsive force on the i-th atom
c                         (reduced units: eV/A= 1.602177E-4 dyne)
c     - fhfx/y/z(i)     : x,y,z component of the Hellmann-Feynman force over i-th atom
c                         (reduced units: eV/A= 1.602177E-4 dyne) 
c     - ftotx/y/z(i)    : x,y,z component of the total force over i-th atom
c                         (reduced units: eV/A= 1.602177E-4 dyne) 
c     - nstep           : number of time-steps of the simulation
c     - dt              : time step
c                         (reduced units: 1 time-step = 1.018050697E-14 sec)
c     - shift           : shift from ideal lattice position (see subroutine crystal)
c                         (percentage over 1nn distance in crystalline silicon)
c     - iseedp          : seed of random generation (see subroutine init_pos)
c     - iseedv          : seed of random generation (see subroutine init_pos)
c     - cut             : cutoff for interactions
c                         (reduced units: A)
c     - bcut            : cutoff for the computation of the bond-angle distibution 
c                         function and atomic coordination number (see subroutine 
c                         nearangle)
c                         (reduced units: A) 
c
c     INPUT/OUTPUT files
c     ------------------ 
c
c     - finp            : input filename (containing atomic positions, velocities and
c                         accelerations of all particles in the simulation box, as 
c                         calculated in the last time-step of a previous simulation)
c     - flast           : output filename (containing atomic positions, velocities
c                         and accelerations of all particles in the simulation box, as 
c                         calculated in the last time-step of the present simulation)
c     - fcorr           : filename for pair correlation function and radial distribution 
c                         function
c     - fbond           : filename for bond angle distribution 
c     - fnear           : filename for atomic coordination number 
c     - fdisp           : filename for atomic mean square displacement
c     - feval           : filename for total electronic density of states
c     - ftp             : filename for temperature
c     - ferg            : filename for kinetic, potential, band-structure and total energy
c     - fpos            : filename for atomic trajectories
c     - fvel            : filename for atomic velocities
c***************************************************************************
c***************************************************************************
c 
c
c
c                               ... PROGRAM STARTS ...
c
c
c
c***************************************************************************
c***************************************************************************
c 
c     Include files
c
      include 'param.inc'
      include 'pos.inc'
      include 'vel.inc'
      include 'acc.inc'
      include 'dist.inc'
      include 'latt.inc'
      include 'tb.inc'
      include 'frep.inc'
      include 'fhf.inc'
      include 'ftot.inc'
      include 'erg.inc'
      include 'eigen.inc'
      include 'func.inc'
      include 'lst.inc'
      include 'simul.inc'
      include 'start.inc'
      include 'potz.inc'
c
      integer iseedp, iseedv      
      real*8 tempf, dtemp, scltmp
      real*8 cut,bcut
      character*80 label
      character*15 finp,fpos,flast,fvel,ftp,ferg,flength
      character*15 fcorr,fbond,fnear,fdisp,feval
c
c     Initialize
c
      do 999 i=1,n
       do 998 j=1,n
        xx(i,j)=0.d0
        yy(i,j)=0.d0
        zz(i,j)=0.d0
        rr2(i,j)=0.d0
  998  continue
  999 continue
c
c     Open input file TBMD.IN
c
      open(unit=9,file='tbmd.in',status='unknown',form='formatted')
c
c     Read input data from file TBMD.IN
c
      read(9,*) label
      read(9,*) nve
      read(9,*) label
      read(9,*) nstart
      read(9,*) label
      read(9,*) finp
      read(9,*) label
      read(9,*) nstep
      read(9,*) label
      read(9,*) nwrp00
      read(9,*) label
      read(9,*) nwrp01
      read(9,*) label
      read(9,*) nwrpos
      read(9,*) label
      read(9,*) fpos
      read(9,*) label
      read(9,*) nwrv00
      read(9,*) label
      read(9,*) nwrv01
      read(9,*) label
      read(9,*) nwrvel
      read(9,*) label
      read(9,*) fvel
      read(9,*) label
      read(9,*) nwrtp
      read(9,*) label
      read(9,*) ftp
      read(9,*) label
      read(9,*) nwrerg
      read(9,*) label
      read(9,*) ferg
      read(9,*) label
      read(9,*) nlast
      read(9,*) label
      read(9,*) flast
      read(9,*) label
      read(9,*) ndisp
      read(9,*) label
      read(9,*) ndisp0
      read(9,*) label
      read(9,*) ndispf
      read(9,*) label
      read(9,*) fdisp
      read(9,*) label
      read(9,*) ncorr
      read(9,*) label
      read(9,*) ncorr0
      read(9,*) label
      read(9,*) ncorrf
      read(9,*) label
      read(9,*) fcorr
      read(9,*) label
      read(9,*) nbond
      read(9,*) label
      read(9,*) nbond0
      read(9,*) label
      read(9,*) nbondf
      read(9,*) label
      read(9,*) fbond
      read(9,*) label
      read(9,*) fnear
      read(9,*) label
      read(9,*) nlength 
      read(9,*) label
      read(9,*) nlength0
      read(9,*) label
      read(9,*) nlengthf
      read(9,*) label
      read(9,*) flength
      read(9,*) label
      read(9,*) neval
      read(9,*) label
      read(9,*) neval0
      read(9,*) label
      read(9,*) nevalf
      read(9,*) label
      read(9,*) feval
      read(9,*) label
      read(9,*) itype
      read(9,*) label
      read(9,*) label
      read(9,*) l0 
      read(9,*) label
      read(9,*) dt
      read(9,*) label
      read(9,*) cut
      read(9,*) label
      read(9,*) bcut
      read(9,*) label
      read(9,*) tempf
c
c     If nstart=0 atomic positions and velocities must be 
c     initially assigned. 
c     If nstart=1 atomic positions, velocities and accelerations are 
c     read from file finp 
c
      if ( nstart .eq. 0 ) then
        read(9,*) label
        read(9,*) shift
        read(9,*) label
        read(9,*) iseedp
        read(9,*) label
        read(9,*) iseedv
        read(9,*) label
        read(9,*) temp0
      else      
        open(unit=98,file=finp,status='unknown',form='formatted')
        read(98,*) temp0
        read(98,*) kintot,ebstot,utot,energy
        do 10 i=1,n
          read(98,*) jdummy
          read(98,*) x(i),y(i),z(i)
          read(98,*) vx(i),vy(i),vz(i)
          read(98,*) axold(i),ayold(i),azold(i)
10      continue
c
        temp=temp0
        tempi=temp0
c 
      endif
c      
c     Subroutine CRYSTAL gives lattice parameters, repulsive potential
c     and tight-binding parameters according for the selected element.
c
      call crystal(itype)
c
      if ( nstart .eq. 0 ) then
c
c     Subroutine INIT_POS computes initial lattice positions.
c
       call init_pos(iseedp)
c
c     Uncomment the following 5 lines if initial positions
c     are read from file ******
c
c1       open(unit=37,file='******',status='unknown')
c2       do i=1,n
c3          read (37,*) x(i),y(i),z(i)
c4       enddo
c5       close(unit=37)
c
c     Subroutine INIT_VEL computes initial velocities corresponding to 
c     the initial temperature temp0.
c
        call init_vel(iseedv)
c
      endif
c
c     Open output files. 
c
      open(unit=97,file=fpos,status='unknown',form='unformatted')
      write(97) n,nwrpos,nstep
      open(unit=96,file=fvel,status='unknown',form='unformatted')
      write(96) n,nwrvel,nstep
      open(unit=87,file=ftp,status='unknown',form='formatted')
      open(unit=84,file=ferg,status='unknown',form='formatted')
      if ( ncorr .eq. 0 ) goto 14
      open(unit=94,file=fcorr,status='unknown',form='formatted')
14    if ( nbond .eq. 0 ) goto 16     
      open(unit=91,file=fbond,status='unknown',form='formatted')
      open(unit=88,file=fnear,status='unknown',form='formatted')
16    if (nlength.eq.0) goto 19 
      open(unit=34,file=flength,status='unknown',form='formatted')   
19    if ( ndisp .eq. 0 ) goto 17
      open(unit=83,file=fdisp,status='unknown',form='formatted')
17    if ( neval .eq. 0 ) goto 18
      open(unit=86,file=feval,status='unknown',form='formatted')      
c
c     Define: - the cutoff for interactions (cutoff) 
c             - the cutoff for subroutine nearangle (bcutof)
c             - dt^2 and dt/2
c
18    cutoff  = cut**2
      bcutof  = bcut**2
      dtsqr   = dt*dt/2.d0
      dthalf  = dt/2.d0
c
c     Periodic boundary conditions.
c
        do 20 i=1,n
          x(i)=x(i)-cell*(dint(x(i)/cell+1.d0)-1.d0)
          y(i)=y(i)-cell*(dint(y(i)/cell+1.d0)-1.d0)
          z(i)=z(i)-cell*(dint(z(i)/cell+1.d0)-1.d0)
20      continue
c  
c     Compute the relative position and the relative
c     squared distance of any atom to the nearest image of any other
c     one, according to periodic boundary conditions.
c
      do 50 i=1,n
       do 60 j=i+1,n
        xx(i,j)= x(j)-x(i)
        xx(i,j)=xx(i,j)-cell*(dint((xx(i,j)/cell2+3.d0)/2.d0)-1.d0)
        yy(i,j)= y(j)-y(i)
        yy(i,j)=yy(i,j)-cell*(dint((yy(i,j)/cell2+3.d0)/2.d0)-1.d0)
        zz(i,j)= z(j)-z(i)  
        zz(i,j)=zz(i,j)-cell*(dint((zz(i,j)/cell2+3.d0)/2.d0)-1.d0)
        rr2(i,j)= xx(i,j)**2 + yy(i,j)**2 + zz(i,j)**2
c
        xx(j,i)=-xx(i,j)
        yy(j,i)=-yy(i,j)
        zz(j,i)=-zz(i,j)
        rr2(j,i)= rr2(i,j)
60     continue
50    continue
c
c     Compute Verlet list 
c
      do 63 i=1,n
        ilst=0
        jlst=0
        do 66 j=1,n
          if ( i .eq. j ) goto 66
          if ( rr2(i,j) .gt. cutoff ) goto 66
          ilst= ilst+1
          list(i,ilst)=j
          if ( i .gt. j ) goto 66
          jlst=jlst+1
          list1(i,jlst)=j
66      continue
        nlst(i)=ilst
        nlst1(i)=jlst
63    continue
c
      if ( nstart .eq. 0 ) then
c
      temp= temp0
c
c     Subroutine REPULS computes repulsive forces.
c
      call repuls(itype)
c
c     Subroutine HTB computes and diagonalizes tight-binding hamiltonian
c     matrix.
c   
      call htb
c
c     Subroutine FEYNMAN computes Hellmann-Feynman forces between atoms.
c      
      call feynman
c
c     Compute the total forces and accelerations.
c
      do 70 i=1,n
        ftotx(i)= frepx(i)+fhfx(i)
        axold(i)= ftotx(i)/mass
        ftoty(i)= frepy(i)+fhfy(i)
        ayold(i)= ftoty(i)/mass
        ftotz(i)= frepz(i)+fhfz(i)
        azold(i)= ftotz(i)/mass
70    continue
c
c     Compute istantaneous temperature.
c
      call temper
      temp = tempi
c 
c     Compute total energy per atom
c
      energy = utot+ebstot+kintot
c
      endif
c
      istep=0
c
c     Compute mean square displacement
c
      if ( ndisp .eq. 0 ) goto 81
      if ( ndisp0 .gt. 0 ) goto 81
c
      call meandisp(istep,ndisp0)
c
c     Output data.
c
81    write(87,901) istep,temp,temp
      write(84,902) istep,kintot,utot,ebstot,energy             
c
      if ( nstep .eq. 0 ) goto 100
c
c     Compute cooling/heating rate [ if (tempf.ne.temp0) ].
c
      dtemp= (tempf-temp0)/dfloat(nstep)
c
c     Molecular dynamics do-loop
c
      do 110 istep=1,nstep
c
c     Decrease/increase temperature [ if (tempf.ne.temp0) ].
c
      temp= temp+dtemp
c
c     Rescale atomic velocities to keep temperature constant
c     (if nve>1).
c
      if (nve .ne. 1) then
         scltmp= dsqrt(temp/tempi)
         do 112 i=1,n
           vx(i)= vx(i)*scltmp
           vy(i)= vy(i)*scltmp
           vz(i)= vz(i)*scltmp
112      continue
      end if
c
c     Compute mean square displacement.
c
      if ( ndisp .eq. 0 ) goto 105
      if ( istep .lt. ndisp0 ) goto 105
      if ( istep .gt. ndispf ) goto 105
c
        call meandisp(istep,ndisp0)
c
c     Subroutine MD  updates atomic positions, velocities 
c     and accelerations .
c
105   call md(itype)
c
c      Subroutine DOSENERGY computes the electronic density of states.
c
      if ( neval .eq. 0 ) goto 119
      if ( istep .lt. neval0 ) goto 119
      if ( istep .gt. nevalf ) goto 119
c
        call dosenergy(istep,neval0,nevalf)
c
c     Subroutine CORRELATE computes pair correlation function and
c     radial distribution function.
c
119   if ( ncorr .eq. 0 ) goto 120
      if ( istep .lt. ncorr0 ) goto 120
      if ( istep .gt. ncorrf ) goto 120
c
        call correlate(istep,ncorr0,ncorrf)
c
c     Subroutine NEARANGLE computes bond angle distribution and
c     atomic coordination number. 
c
120   if ( nbond .eq. 0 ) goto 130
      if ( istep .lt. nbond0 ) goto 130
      if ( istep .gt. nbondf ) goto 130
c
        call nearangle(istep,nbond0,nbondf)
c
c     Subroutine BONDLENGTH computes bond length distribution
c
130   if ( nlength .eq. 0 ) goto 140
      if ( istep .lt. nlength0 ) goto 140
      if ( istep .gt. nlengthf ) goto 140
c
        call bondlength(istep,nlength0,nlengthf,bcutof)
c
c     Compute total energy per atom
c
140   energy= utot+ebstot+kintot
c
c     Save atomic trajectories.
c
      if ( istep .lt. nwrp00 ) goto 1111
      if ( istep .gt. nwrp01 ) goto 1111
      if ( istep .eq. nwrpos*(istep/nwrpos) ) then
c
        do 150 i=1,n
          write(97) x(i),y(i),z(i)
150     continue
c
      endif
c
c     Save intermediate configuration.
c
1111  if ( istep .eq. nlast*(istep/nlast) ) then
c
      open (unit=99,file=flast,status='unknown',form='formatted')
      write(99,*) temp
      write(99,904) kintot, ebstot, utot, energy
      do 167 i=1,n
        write(99,*) i
        write(99,*) x(i),y(i),z(i)
        write(99,*) vx(i),vy(i),vz(i)
        write(99,*) axold(i),ayold(i),azold(i)     
c
167   continue   
c
      close(unit=99)
c      
      endif
c
c     Save atomic velocities.
c
      if ( istep .lt. nwrv00 ) goto 1112
      if ( istep .gt. nwrv01 ) goto 1112
      if ( istep .eq. nwrvel*(istep/nwrvel) ) then
c
        do 151 i=1,n
          write(96) vx(i),vy(i),vz(i)
151     continue
c
      endif
c
1112  if ( istep .eq. nwrtp*(istep/nwrtp)) then    
c
c     Write reference and istantaneous temperature.
c
        write(87,901) istep,temp,tempi
c
      endif
c
      if ( istep .eq. nwrerg*(istep/nwrerg)) then
c
c     Write istantaneous kinetic, repulsive, band-structure and total energy.
c
        write(84,902) istep,kintot,utot,ebstot,energy
c
      endif             
c
110   continue
c
c     Save last configuration.
c
      open (unit=99,file=flast,status='unknown',form='formatted')
      write(99,*) temp
      write(99,904) kintot, ebstot, utot, energy
      do 160 i=1,n
        write(99,*) i
        write(99,*) x(i),y(i),z(i)
        write(99,*) vx(i),vy(i),vz(i)
        write(99,*) axold(i),ayold(i),azold(i)
160   continue   
c
901   format (i6,3(1x,g15.8))
902   format (i6,4(1x,g15.8))
904   format (4(1x,g15.8))
c
100   stop
      end
c**************************************************************************** 
      subroutine correlate(istep,ncorr0,ncorrf)
c**************************************************************************** 
c     RELEASE TBMD.2.0
c     Last revision: November 21, 1996 
c****************************************************************************
c     Contact author:    Luciano Colombo
c                        e-mail:   colombo@mi.infn.it
c****************************************************************************
c
c     Subroutine CORRELATE computes pair correlation function and
c     radial distribution of the system.
c
c     Variables
c
c     - nsudd         : number of different intervals for computing radial 
c                       distribution
c     - xx/yy/zzm     : vector of relative distance between atoms
c     - rrm           : relative distance between atoms
c     - corr(nsudd)   : radial distribution function
c     - avcorr(nsudd) : pair correlation function 
c
c****************************************************************************
c
c     Include files and parameters
c
      include 'param.inc'
      parameter ( nsuddc= 300 )
      include 'dist.inc'
      include 'latt.inc'
      include 'pos.inc'
c
      real*8 xxm, yym, zzm, rrm
      real*8 xxn, yyn, zzn
      real*8 rmin, rmax, rsudd, rpos, rlim
      real*8 avcorr(nsuddc), corr(nsuddc)
      real*8 volume, dens, crown
c
      data avcorr/nsuddc*0.d0/
      data corr/nsuddc*0.d0/
c
c     Range for radial distribution
c
      rmax= cell/2.d0
      rmin= 0.d0 
c
c     Widht of intervals 
c 
      rsudd= ( rmax - rmin )/ dfloat(nsuddc)
c
c     Calculation of radial distribution function
c
      do 10 i=1,n
        do 20 j=i+1,n
          xxn= xx(i,j)
          yyn= yy(i,j)
          zzn= zz(i,j)
c
          do 30 i1=0,1
            xxm= xxn-dfloat(i1)*(xxn/dabs(xxn))*cell
            do 40 i2=0,1
              yym= yyn-dfloat(i2)*(yyn/dabs(yyn))*cell
              do 50 i3=0,1
                zzm= zzn-dfloat(i3)*(zzn/dabs(zzn))*cell
                rrm= dsqrt(xxm**2+yym**2+zzm**2)
                if ( rrm .ge. rmax ) goto 50
                npos= int((rrm-rmin)/rsudd)+1
                corr(npos)= corr(npos)+2.d0
50            continue
40          continue
30        continue
c
20      continue
10    continue
c
c     Output 
c
      if ( istep .eq. ncorrf ) then
      volume = cell**3
      dens   = 4.d0*dacos(-1.d0)/3.d0*dfloat(n)/volume
        do 70 isudd=1,nsuddc
          rlim= rmin+rsudd*(dfloat(isudd)-1.0d0)
          rpos= rmin+rsudd*(dfloat(isudd)-0.5d0)
          crown= rsudd*(rsudd**2+3.d0*(rlim*(rlim+rsudd)))*dens
          corr(isudd)= corr(isudd)/dfloat(ncorrf-ncorr0+1)/dfloat(n)
          avcorr(isudd)= corr(isudd)/crown
          write(94,*) rpos,avcorr(isudd),corr(isudd)
70      continue
      endif
c
      return
      end
c****************************************************************************
      subroutine crystal(itype)
c****************************************************************************
c     RELEASE TBMD.2.0
c     Last revision: November 21, 1996 
c****************************************************************************
c     Contact author:   L.Colombo
c                       colombo@mi.infn.it
c****************************************************************************
c
c     Subroutine CRYSTAL computes lattice parameters and defines
c     both repulsive potential and tight-binding parameters for the selected material
c
c     Variables
c
c     - itype       : 1 ==> Carbon
c                     2 ==> Silicon
c                     >2 ==> not implemented 
c     - ncell       : number of replicas along x,y and z of the
c                     conventional diamond cell in the simulation box
c     - mass        : atomic mass (amu = 1.6605402E-24 g)
c     - r0          : 1nn equilibrium distance in the diamond lattice (A)
c                     [this value enters in the scaling function and does NOT
c                      depend on the system density]
c     - l0          : lenght of the cubic conventional diamond cell (A)
c     - cell        : lenght of simulation box (A)
c     - ess         : on-site s orbital  (eV)
c     - epp         : on-site p orbital  (eV)
c     - sss         : two center s-s hopping integral (eV)
c     - sps         : two center s-p hopping integral (eV)
c     - pps         : two center p-p hopping integral (sigma) (eV)
c     - ppp         : two center p-p hopping integral (pi) (eV)
c
c     - d0,phi0     : these parameters are defined only for carbon 
c
c****************************************************************************
c
c     Include files
c
      include 'param.inc'
      include 'latt.inc'
      include 'tb.inc'
      include 'potz.inc'
      include 'pos.inc'
      include 'start.inc'
c
      kboltz=8.6173855d-5
c
c     Select material
c
      if ( itype .eq. 1 ) goto 10
      if ( itype .eq. 2 ) goto 20
      if ( itype .gt. 2 ) then
         write(6,*) '  ' 
         write(6,*) '>>>> WARNING: wrong itype  !!!'
         write(6,*) '>>>> Program stop'
      end if
c
c     *******************
c     *****CARBON********
c     *******************
c
10      mass = 12.011d0
        cell = l0*dfloat(ncell)
        r0   = 1.536329d0
c
c     parameters for hopping integrals and their scaling functions
c     1: sss ; 2: sps; 3: pps; 4: ppp
c
        ess = -2.99d0
        epp =  3.71d0
        sss = -5.0d0
        sps =  4.7d0
        pps =  5.5d0
        ppp = -1.55d0
c
        enc1 = 6.5d0
        rc1  = 2.18d0
        enc2 = 6.5d0
        rc2  = 2.18d0
        enc3 = 6.5d0
        rc3  = 2.18d0
        enc4 = 6.5d0
        rc4  = 2.18d0
c
c XWCH parameters for repulsive potential
c
        em   =  3.30304d0
        emc  =  8.6655d0
        dc   =  2.1052d0
        d0   =  1.64d0
        phi0 =  8.18555d0
        c0   = -2.5909765118191d0
        c1   =  0.5721151498619d0
        c2   = -1.7896349903996d-3
        c3   =  2.3539221516757d-5
        c4   = -1.24251169551587d-7
c
      goto 100
c
c     *******************
c     ***** SILICON *****
c     *******************
c
20      mass = 28.0855
        cell = l0*dfloat(ncell)
        r0   = 2.360352d0
c
c     ess and epp are shifted with respect to the original paper
c     so that the topmost valence band falls at 0eV.
c     shift=-0.46363896
c
c     parameters for hopping integrals and their scaling functions
c     1: sss ; 2: sps; 3: pps; 4: ppp
c
        ess = -5.25d0-0.46363896d0
        epp =  1.20d0-0.46363896d0
        sss = -2.038d0
        sps =  1.745d0
        pps =  2.75d0
        ppp = -1.075d0
c
        enc1 = 9.5d0
        rc1  = 3.4d0
        enc2 = 8.5d0
        rc2  = 3.55d0
        enc3 = 7.5d0
        rc3  = 3.7d0
        enc4 = 7.5d0
        rc4  = 3.7d0
c
c KBWHS parameters for repulsive potential
c (n.b. d0 and phi0 are NOT used. Their value is fixed to a
c       random number. The constant c0 is not used: c0=0.d0
c       SEE SUBROUTINE REPULS FOR FURTHER DETAILS)
c
        em   =  6.8755d0
        emc  = 13.017d0
        dc   =  3.66995d0
        d0   =  99999.9999d0
        phi0 =  99999.9999d0 
        c0   =  0.d0
        c1   =  2.1604385d0
        c2   = -0.1384393d0
        c3   =  5.8398423d-3
        c4   = -8.0263577d-5
c
      goto 100
c
c     calculated data
c
100   r02= r0*r0
      cell2= cell/2.d0
      psp= pps-ppp
c
      return
      end
c****************************************************************************
      subroutine dosenergy(istep,neval0,nevalf)
c**************************************************************************** 
c     RELEASE TBMD.2.0
c     Last revision: November 21, 1996 
c****************************************************************************
c     Contact author:    Luciano Colombo
c                        e-mail:   colombo@mi.infn.it
c****************************************************************************
c
c     Subroutine DOSENERGY  computes the electronic density of 
c     states.
c
c****************************************************************************
c
c     Include files
c
      include 'param.inc'
      parameter ( nsudde= 300 )
      include 'eigen.inc'
c
      real*8 edos(nsudde)
      real*8 emin, emax, esudd, epos
c
      data edos / nsudde * 0.d0 /
c
c     ATTENTION: the following values for emin and emax
c                can be adjusted for plotting 
c
      emin=-30.d0
      emax= 30.d0
      esudd= ( emax-emin )/dfloat(nsudde)
c
      do 10 i=1,n4
        npos= int (( eval(i)-emin )/esudd )+1
        edos(npos)= edos(npos)+1.d0
10    continue
c
c     Output
c
      if (istep .eq. nevalf ) then
c
        do 20 isudd=1,nsudde
          epos= emin+esudd*(dfloat(isudd)-0.5d0)
          edos(isudd)= edos(isudd)/dfloat(nevalf-neval0+1)
          write(86,*) epos, edos(isudd)
20      continue
c
      endif
c
      return
      end
c****************************************************************************
      subroutine feynman     
c**************************************************************************** 
c     RELEASE TBMD.2.0
c     Last revision: November 21, 1996 
c****************************************************************************
c     Contact author:    Luciano Colombo 
c                        colombo@mi.infn.it 
c****************************************************************************
c
c     Subroutine FEYNMAN computes the Hellmann-Feynman contribution to the
c     atomic forces
c
c      This routine has been optimized 
c
c****************************************************************************
c
c     Include files
c
      include 'param.inc'
      include 'dist.inc'
      include 'eigen.inc'
      include 'latt.inc'
      include 'lst.inc'
      include 'fhf.inc'
      include 'potz.inc'
      include 'tb.inc'
c
      real*8 fhsumx, fhsumy, fhsumz
      real*8 xxn, yyn, zzn, rr2n, rrn
      real*8 scale1, scale2, scale3, scale4
      real*8 dsca1, dsca2, dsca3, dsca4
      real*8 dscalx1, dscaly1, dscalz1
      real*8 dscalx2, dscaly2, dscalz2
      real*8 dscalx3, dscaly3, dscalz3
      real*8 dscalx4, dscaly4, dscalz4
      real*8 fhmat(4,4)
      real*8 fsmx, fsmy, fsmz
      real*8 sum
      real*8 fhffx(n,n), fhffy(n,n), fhffz(n,n)
c
      do 10 i=1,n
        fhsumx= 0.d0
        fhsumy= 0.d0
        fhsumz= 0.d0
c
        do 20 ji=1,nlst1(i)
          j=list1(i,ji)
          rr2n= rr2(i,j)
          rrn= dsqrt(rr2n)
          xxn= xx(i,j)/rrn
          yyn= yy(i,j)/rrn
          zzn= zz(i,j)/rrn
c
c     Scaling functions for hopping integrals
c
          scale1= (r0/rrn)**2 / rrn *
     *             dexp( 2.d0 *( - (rrn/rc1)**enc1 + (r0/rc1)**enc1 ))           
          scale2= (r0/rrn)**2 / rrn *
     *             dexp( 2.d0 *( - (rrn/rc2)**enc2 + (r0/rc2)**enc2 ))           
          scale3= (r0/rrn)**2 / rrn *
     *             dexp( 2.d0 *( - (rrn/rc3)**enc3 + (r0/rc3)**enc3 ))           
          scale4= (r0/rrn)**2 / rrn *
     *             dexp( 2.d0 *( - (rrn/rc4)**enc4 + (r0/rc4)**enc4 ))           
c
c     Derivative of the scaling law
c
          dsca1= -2.d0 * scale1 * ( 1.d0 + enc1* (rrn/rc1)**enc1 ) 
          dscalx1= dsca1 * xxn 
          dscaly1= dsca1 * yyn
          dscalz1= dsca1 * zzn
          dsca2= -2.d0 * scale2 * ( 1.d0 + enc2* (rrn/rc2)**enc2 ) 
          dscalx2= dsca2 * xxn 
          dscaly2= dsca2 * yyn
          dscalz2= dsca2 * zzn
          dsca3= -2.d0 * scale3 * ( 1.d0 + enc3* (rrn/rc3)**enc3 ) 
          dscalx3= dsca3 * xxn 
          dscaly3= dsca3 * yyn
          dscalz3= dsca3 * zzn
          dsca4= -2.d0 * scale4 * ( 1.d0 + enc4* (rrn/rc4)**enc4 ) 
          dscalx4= dsca4 * xxn 
          dscaly4= dsca4 * yyn
          dscalz4= dsca4 * zzn
c 
c     Matrix derivative of tight-binding hamiltonian
c
c
c     Compute x-component
c
          fhmat(1,1)=  sss * dscalx1
          fhmat(1,2)=  sps * ( xxn * dscalx2 +
     *                      scale2 * ( 1.d0 - xxn**2 ) ) 
          fhmat(1,3)=  sps * ( yyn * dscalx2 -
     *                      scale2 * yyn * xxn ) 
          fhmat(1,4)=  sps * ( zzn * dscalx2 -
     *                      scale2 * zzn * xxn )       
c
          fhmat(2,1)= - fhmat(1,2)
          fhmat(2,2)=  ( pps * dscalx3 * xxn**2 + 
     1                      ppp * dscalx4 * ( 1.d0 - xxn**2 ) ) +
     2                      2.d0 * ( pps * scale3 - ppp * scale4 ) *
     3                      xxn * ( 1.d0 - xxn**2 )   
          fhmat(2,3)=  ( pps * dscalx3 - ppp * dscalx4 )*xxn*yyn 
     1                      + ( pps * scale3 - ppp * scale4)
     2                      * ( yyn - 
     3                      2.d0 * xxn**2 * yyn ) 
          fhmat(2,4)=  ( pps * dscalx3 - ppp * dscalx4 )*xxn*zzn 
     1                      + ( pps * scale3 - ppp * scale4 )
     2                      * ( zzn -
     3                      2.d0 * xxn**2 * zzn ) 
c
          fhmat(3,1)= - fhmat(1,3)
          fhmat(3,2)=   fhmat(2,3)
          fhmat(3,3)=  ( pps * dscalx3 * yyn**2 +
     1                      ppp * dscalx4 * ( 1.d0 - yyn**2 ) ) -
     2                      2.d0 * ( pps * scale3 - ppp * scale4 ) *
     3                      yyn**2 * xxn
          fhmat(3,4)=  ( pps * dscalx3 - ppp * dscalx4 ) *yyn*zzn 
     1                      - ( pps * scale3 - ppp * scale4 ) * 
     2                      2.d0 *  
     3                      xxn * yyn * zzn        
c
          fhmat(4,1)= - fhmat(1,4)
          fhmat(4,2)=   fhmat(2,4)
          fhmat(4,3)=   fhmat(3,4)
          fhmat(4,4)=  ( pps * dscalx3 * zzn**2 +
     1                      ppp * dscalx4 * ( 1.d0 - zzn**2 ) ) -
     2                      2.d0 * ( pps * scale3  - ppp * scale4 ) *
     3                      zzn**2 * xxn
c
          fsmx = 0.d0
          do 30 im=1,4
            do 40 jm=1,4
              i1= 4*(i-1)+im
              j1= 4*(j-1)+jm
c
c We avoid useless computation of matrix products by exploiting
c the step-like behaviour of the T=OK Fermi-Dirac distribution
c function.
c Since occup(i)=0.d0 if i>n4/2, then the variable sum is computed
c only on the occupied part of the spectrum.
c This implementation holds ONLY if:
c  - the occupancy is 2 for ANY filled state
c  - n4 is even (i.e. an even number of orbitals per atom)
c
              sum = 0.d0
              do 50 k=1,n4/2
                sum=  sum + occup(k) * h(i1,k) * h(j1,k)
50            continue
c
              fsmx= fsmx + sum * fhmat(im,jm)
c
40          continue
30        continue
c
c     Compute y-component
c
          fhmat(1,1)=  sss * dscaly1
          fhmat(1,2)=  sps * ( xxn * dscaly2 -
     *                      scale2 * xxn * yyn ) 
          fhmat(1,3)=  sps * ( yyn * dscaly2 +
     *                      scale2 * ( 1.d0 - yyn ** 2 ) ) 
          fhmat(1,4)=  sps * ( zzn * dscaly2 -
     *                      scale2 * zzn * yyn )       
c
          fhmat(2,1)= - fhmat(1,2)
          fhmat(2,2)=  ( pps * dscaly3 * xxn**2 + 
     1                      ppp * dscaly4 * ( 1.d0 - xxn**2 ) ) -
     2                      2.d0 * ( pps * scale3 - ppp * scale4 ) *
     3                      xxn **2 * yyn    
          fhmat(2,3)=  ( pps * dscaly3 - ppp * dscaly4 ) *xxn*yyn 
     1                      + ( pps * scale3 - ppp * scale4)
     2                      * ( xxn - 
     3                      2.d0 * yyn**2 * xxn ) 
          fhmat(2,4)=  ( pps * dscaly3 - ppp * dscaly4 ) 
     1                      * xxn * zzn 
     2                      - 2.d0 * ( pps * scale3 - ppp * scale4 )
     3                      * xxn * yyn * zzn  
c
          fhmat(3,1)= - fhmat(1,3)
          fhmat(3,2)=   fhmat(2,3)
          fhmat(3,3)=  ( pps * dscaly3 * yyn**2 +
     1                      ppp * dscaly4 * ( 1.d0 - yyn**2 ) ) +
     2                      2.d0 * ( pps * scale3 - ppp * scale4 ) *
     3                      yyn* ( 1.d0 - yyn**2 )
          fhmat(3,4)=  ( pps * dscaly3 - ppp * dscaly4 ) *yyn*zzn 
     1                      + ( pps * scale3 - ppp * scale4 ) * 
     2                      ( zzn - 2.d0 *  
     3                        yyn**2 * zzn )        
c
          fhmat(4,1)= - fhmat(1,4)
          fhmat(4,2)=   fhmat(2,4)
          fhmat(4,3)=   fhmat(3,4)
          fhmat(4,4)=  ( pps * dscaly3 * zzn**2 +
     1                      ppp * dscaly4 * ( 1.d0 - zzn**2 ) ) -
     2                      2.d0 * ( pps * scale3  - ppp * scale4 ) *
     3                      zzn**2 * yyn
c
          fsmy = 0.d0
          do 31 im=1,4
            do 41 jm=1,4
              i1= 4*(i-1)+im
              j1= 4*(j-1)+jm
c
c We avoid useless computation of matrix products by exploiting
c the step-like behaviour of the T=OK Fermi-Dirac distribution
c function.
c Since occup(i)=0.d0 if i>n4/2, then the variable sum is computed
c only on the occupied part of the spectrum.
c This implementation holds ONLY if:
c  - the occupancy is 2 for ANY filled state
c  - n4 is even (i.e. an even number of orbitals per atom)
c
              sum= 0.d0
              do 51 k=1,n4/2
                sum=  sum + occup(k) * h(i1,k) * h(j1,k)
51            continue
c
              fsmy= fsmy + sum * fhmat(im,jm)
c
41          continue
31        continue
c
c     Compute z-component
c
          fhmat(1,1)=  sss * dscalz1
          fhmat(1,2)=  sps * ( xxn * dscalz2 -
     *                      scale2 * xxn * zzn )
          fhmat(1,3)=  sps * ( yyn * dscalz2 -
     *                      scale2 * yyn * zzn ) 
          fhmat(1,4)=  sps * ( zzn * dscalz2 +
     *                      scale2 * ( 1.d0 - zzn **2 ) )       
c
          fhmat(2,1)= - fhmat(1,2)
          fhmat(2,2)=  ( pps * dscalz3 * xxn**2 + 
     1                      ppp * dscalz4 * ( 1.d0 - xxn**2 ) ) -
     2                      2.d0 * ( pps * scale3 - ppp * scale4 ) *
     3                      xxn **2 * zzn   
          fhmat(2,3)=  ( pps * dscalz3 - ppp * dscalz4 ) *xxn*yyn 
     1                      - 2.d0 * ( pps * scale3 - ppp * scale4)
     2                      *  
     3                      xxn* yyn * zzn  
          fhmat(2,4)=  ( pps * dscalz3 - ppp * dscalz4 ) *xxn*zzn 
     1                      + ( pps * scale3 - ppp * scale4 )
     2                      * ( xxn -
     3                      2.d0 * zzn**2 * xxn ) 
c
          fhmat(3,1)= - fhmat(1,3)
          fhmat(3,2)=   fhmat(2,3)
          fhmat(3,3)=  ( pps * dscalz3 * yyn**2 +
     1                      ppp * dscalz4 * ( 1.d0 - yyn**2 ) ) -
     2                      2.d0 * ( pps * scale3 - ppp * scale4 ) *
     3                      yyn**2 * zzn
          fhmat(3,4)=  ( pps * dscalz3 - ppp * dscalz4 )*yyn*zzn 
     1                      + ( pps * scale3 - ppp * scale4 ) *(yyn -  
     2                      2.d0 *  
     3                      zzn ** 2 * yyn )        
c
          fhmat(4,1)= - fhmat(1,4)
          fhmat(4,2)=   fhmat(2,4)
          fhmat(4,3)=   fhmat(3,4)
          fhmat(4,4)=  ( pps * dscalz3 * zzn**2 +
     1                      ppp * dscalz4 * ( 1.d0 - zzn**2 ) ) +
     2                      2.d0 * ( pps * scale3  - ppp * scale4 )*
     3                      zzn * ( 1.d0 - zzn ** 2 ) 
c
          fsmz= 0.d0
c
          do 32 im=1,4
            do 42 jm=1,4
              i1= 4*(i-1)+im
              j1= 4*(j-1)+jm
c
c We avoid useless computation of matrix products by exploiting
c the step-like behaviour of the T=OK Fermi-Dirac distribution
c function.
c Since occup(i)=0.d0 if i>n4/2, then the variable sum is computed
c only on the occupied part of the spectrum.
c This implementation holds ONLY if:
c  - the occupancy is 2 for ANY filled state
c  - n4 is even (i.e. an even number of orbitals per atom)
c
              sum= 0.d0              
              do 52 k=1,n4/2
                sum=  sum + occup(k) * h(i1,k) * h(j1,k)
52            continue
c
              fsmz= fsmz + sum * fhmat(im,jm) 
c
42          continue
32        continue
c
c Compute x-,y-,z-component of the Helmann-Feynman force matrix
c
          fhffx(i,j)=  fsmx
          fhffy(i,j)=  fsmy
          fhffz(i,j)=  fsmz
c
          fhffx(j,i)= -fsmx
          fhffy(j,i)= -fsmy
          fhffz(j,i)= -fsmz
c
20      continue
c
        do 60 ji=1,nlst(i)
          j=list(i,ji)
c
          fhsumx=  fhsumx + fhffx(i,j)                    
          fhsumy=  fhsumy + fhffy(i,j)                    
          fhsumz=  fhsumz + fhffz(i,j)                    
c
60      continue
c
        fhfx(i)= 2.d0 * fhsumx  
        fhfy(i)= 2.d0 * fhsumy  
        fhfz(i)= 2.d0 * fhsumz
c
10    continue
c
      return
      end
c****************************************************************************
      subroutine htb
c**************************************************************************** 
c     RELEASE TBMD.2.0
c     Last revision: November 21, 1996 
c****************************************************************************
c     Contact author:   L. Colombo 
c                        e-mail:   colombo@mi.infn.it 
c****************************************************************************
c
c     Subroutine HTB computes and diagonalizes the tight-binding hamiltonian
c     matrix.
c     LAPACK library for matrix diagonalization is used.
c
c     Optimized use of memory
c   
c     The actual dimension of the hamiltonian matrix to diagonalize
c     are set to (n4+1,n4+1). This allows for avoiding bank conflicts
c     The extrac line and column of h is filled by zero matrix entries,
c     but for the (n4+1,n4+1) element: it is set equal to 100.d0,
c     i.e. a number well outside the typical energy spectrum of the
c     TB matrix.
c
c****************************************************************************
c
c     Include files.
c
      include 'param.inc'
      include 'dist.inc'
      include 'eigen.inc'
      include 'latt.inc'
      include 'tb.inc'
      include 'func.inc'
      include 'lst.inc'
      include 'erg.inc'
      include 'potz.inc'
c
      integer lwork,info
      parameter(lwork=34*(n4+1))
c
      real*8 xxn, yyn, zzn, rr2n, rrn
      real*8 scale1, scale2, scale3, scale4
      real*8 etot
      real*8 work(lwork)
      character*1 jobz,uplo
c
      do 10 i1=1,n4+1
        do 20 i2=1,i1-1
           h(i2,i1)=0.d0
20      continue
10    continue
      h(n4+1,n4+1)=100.d0
c
c     Compute hamiltonian matrix
c
      do 100 i=1,n
c
        i43= 4*i-3
        i42= 4*i-2
        i41= 4*i-1
        i40= 4*i
c
        do 200 ji=1,nlst1(i)
          j= list1(i,ji)
c
          j43= 4*j-3
          j42= 4*j-2
          j41= 4*j-1
          j40= 4*j
c
          rr2n= rr2(i,j)
          rrn= dsqrt(rr2n)
c
          xxn= xx(i,j)/rrn
          yyn= yy(i,j)/rrn
          zzn= zz(i,j)/rrn
c
c     Scaling functions for hopping integrals
c
          scale1= (r0/rrn)**2 *
     *             dexp( 2.d0 *(-(rrn/rc1)**enc1 + (r0/rc1)**enc1))
          scale2= (r0/rrn)**2 *
     *             dexp( 2.d0 *(-(rrn/rc2)**enc2 + (r0/rc2)**enc2))
          scale3= (r0/rrn)**2 *
     *             dexp( 2.d0 *(-(rrn/rc3)**enc3 + (r0/rc3)**enc3))
          scale4= (r0/rrn)**2 *
     *             dexp( 2.d0 *(-(rrn/rc4)**enc4 + (r0/rc4)**enc4))
c
          h(i43,j43)=  sss * scale1
          h(i43,j42)=  sps * xxn * scale2
          h(i43,j41)=  sps * yyn * scale2
          h(i43,j40)=  sps * zzn * scale2
c
          h(i42,j43)= - h(i43,j42)
c
          h(i42,j42)=  ( pps * scale3 * xxn**2 +
     *                   ppp * scale4 * ( 1.d0 - xxn**2 ) )
          h(i42,j41)=  ( pps * scale3 - ppp * scale4 ) * xxn * yyn 
          h(i42,j40)=  ( pps * scale3 - ppp * scale4 ) * xxn * zzn 
c
          h(i41,j43)= - h(i43,j41)
          h(i41,j42)=   h(i42,j41)
c
          h(i41,j41)=  ( pps * scale3 * yyn**2 +
     *                   ppp * scale4 * ( 1.d0 - yyn**2 ) ) 
          h(i41,j40)=  ( pps * scale3 - ppp * scale4 ) * yyn * zzn 
c
          h(i40,j43)= - h(i43,j40)
          h(i40,j42)=   h(i42,j40)
          h(i40,j41)=   h(i41,j40)
c
          h(i40,j40)=  ( pps * scale3 * zzn**2 +
     *                   ppp * scale4 * ( 1.d0 - zzn**2 ) )
c
200     continue
c
        h(i43,i43)=  ess
        h(i42,i42)=  epp
        h(i41,i41)=  epp
        h(i40,i40)=  epp
c
100   continue
c
c     Diagonalization
c 
      jobz='V'
      uplo='U'
      call dsyev(jobz,uplo,n4+1,h,n4+1,eval,work,lwork,info)
c
      etot = 0.d0
c
c     Compute the statistical weight for any energy level.
c     Zero-temperature Fermi-Dirac distribution is assumed.
c
      do 31 i=1,n4+1
         if (i.gt.n4/2) then
            occup(i) = 0.d0
         else
            occup(i) = 2.d0
         end if
31    continue
c
c     Compute band-structure energy
c
      do 30 i=1,n4+1 
         etot = etot + eval(i)*occup(i)
30    continue
c
c     Compute band-structure energy per atom
c
      ebstot = etot / dfloat(n)
c
      return
      end
c****************************************************************************
      subroutine init_pos(iseed)
c**************************************************************************** 
c     RELEASE TBMD.2.0
c     Last revision: November 21, 1996 
c****************************************************************************
c     Contact author:    Luciano Colombo
c                        e-mail:   colombo@mi.infn.it
c****************************************************************************
c
c     Subroutine INIT_POS computes initial positions of n atoms in a  
c     cubic diamond-like simulation box with linear dimension l0*ncell 
c     Ideal positions are affected by a random shift.
c
c     Variables
c
c     - iseed       : seed of random numbers generation
c     - xran        : real*8 random number generated by RAN1
c
c*****************************************************************************
c
c      Include files
c
      include 'param.inc'
      include 'pos.inc'
      include 'latt.inc'
      include 'start.inc'
c
      real*8 xran
      integer iseed
c
c     Inizialize position vectors.
c
      do 5 i=1,n
         x(i)=0.d0
         y(i)=0.d0
         z(i)=0.d0
5     continue
c
c     Compute perfect lattice positions.
c   
      do 30 j3=0,ncell-1
       do 20 j2=0,ncell-1
        do 10 j1=0,ncell-1
         do 12 i2=0,1
          do 11 i1=1,4
c
           jj=8*(j1+ncell*j2+ncell**2*j3)+i1+4*i2   
c
           x(jj)=l0*(dfloat(j1)+dfloat(i1/3)/2.d0+
     *            dfloat(i2)/4.d0)
           y(jj)=l0*(dfloat(j2)+dfloat(i1/2-i1/3)/2.d0+
     *            dfloat(i2)/4.d0)
           z(jj)=l0*(float(j3)+(dfloat(i1/2)/2.d0-dfloat(i1/4))+ 
     *            dfloat(i2)/4.d0)
c
c     Perfect lattice positions are randomly shifted by a maxmimum 
c     amount of r0*shift.
c
           xran=dble(ran1(iseed))
           x(jj)=x(jj)+(2.d0*xran-1.d0)*r0*shift
           xran=dble(ran1(iseed))
           y(jj)=y(jj)+(2.d0*xran-1.d0)*r0*shift
           xran=dble(ran1(iseed))
           z(jj)=z(jj)+(2.d0*xran-1.d0)*r0*shift
c               
11        continue
12       continue
10      continue
20     continue
30    continue
c
      return
      end        
c****************************************************************************
      subroutine init_vel(iseed)
c**************************************************************************** 
c     RELEASE TBMD.2.0
c     Last revision: November 21, 1996 
c****************************************************************************
c     Contact author:    Luciano Colombo
c                        e-mail:   colombo@mi.infn.it
c****************************************************************************
c
c
c     Subroutine INIT_VEL computes initial velocities according to a 
c     pseudo-Maxwell distribution corresponding to temperature temp0.
c     No zero-velocity particles are created.
c
c     Variables
c           
c     - iseed       : seed of random generation of velocities 
c     - xran        : real*8 random number generated by RAN1
c     - temp0       : initial temperature of simulation
c     - temp00      : temperature of random distribution
c     - kintot      : total kinetic energy per atom
c
c****************************************************************************
c
c     Include files
c
      include 'param.inc'
      include 'vel.inc'
      include 'latt.inc'
      include 'func.inc'
      include 'start.inc'
c
      real*8 sclvel
      real*8 segno,caso
      real*8 sumvx, sumvy, sumvz
      real*8 xran
      integer iseed
c
c     Select distribution ( temp0=0 / temp0>0 )
c
      if ( temp0 .eq. 0.d0 ) goto 100
c
c     Compute velocities according to a pseudo-Maxwell
c     distribution at temperature temp00
c
      do 10 j=1,n
         xran=dble(ran1(iseed))
         segno= 2.d0*xran-1.d0
         if ( segno .eq. 0.d0 ) segno=1.d0
         xran=dble(ran1(iseed))
         caso=xran
         if ( caso .eq. 0.d0 ) caso=0.984d0
         vx(j)= segno/dabs(segno)*dsqrt(-dlog(caso))
         xran=dble(ran1(iseed))
         segno= 2.d0*xran-1.d0
         if ( segno .eq. 0.d0 ) segno=-1.d0
         xran=dble(ran1(iseed))
         caso=xran
         if ( caso .eq. 0.d0 ) caso=0.984d0
         vy(j)= segno/dabs(segno)*dsqrt(-dlog(caso))
         xran=dble(ran1(iseed))
         segno= 2.d0*xran-1.d0
         if ( segno .eq. 0.d0 ) segno=1.d0
         xran=dble(ran1(iseed))
         caso=xran
         if ( caso .eq. 0.d0 ) caso=0.984d0
         vz(j)= segno/dabs(segno)*dsqrt(-dlog(caso))
10    continue
c 
c     Compute total momentum
c
      sumvx= 0.d0
      sumvy= 0.d0
      sumvz= 0.d0
      do 12 i=1,n
        sumvx=sumvx+vx(i)
        sumvy=sumvy+vy(i)
        sumvz=sumvz+vz(i)
12    continue
c
c     Correct initial velocities in order to have 
c     zero total momentum.
c
      do 13 i=1,n
        vx(i)=vx(i)-sumvx/dfloat(n)
        vy(i)=vy(i)-sumvy/dfloat(n)
        vz(i)=vz(i)-sumvz/dfloat(n)
13    continue
c
c     Kinetic temperature from above velocities.
c
      call temper 
c
c     Rescale velocities to temperature temp0
c
      sclvel= dsqrt(temp0/tempi)
c
      do 11 i=1,n
         vx(i)=sclvel*vx(i)
         vy(i)=sclvel*vy(i)
         vz(i)=sclvel*vz(i)
11    continue
c
      goto 122
c      
c     If temp0 = 0 Kelvin, then:
c
100   do 20 j=1,n
         vx(j)= 0.d0
         vy(j)= 0.d0
         vz(j)= 0.d0
20    continue
c
122    return
      end
c****************************************************************************
      subroutine md(itype)
c**************************************************************************** 
c     RELEASE TBMD.2.0
c     Last revision: November 21, 1996
c****************************************************************************
c     Contact author:    Luciano Colombo
c                        e-mail:   colombo@mi.infn.it
c****************************************************************************
c
c     Subroutine MD updates atomic positions, velocities and accelerations 
c     according to the velocity Verlet algorithm.   
c
c     Variables (see also main program for futher information)
c
c     - x/y/z(i)      : position of the i-th atom
c     - vx/y/z(i)     : velocity of the i-th atom
c     - ax/y/z(i)     : acceleration of the i-th atom
c     - ax/y/zold(i)  : acceleration of the i-th atom at the previous step
c     - xx/yy/zz(i,J) : relative position of the j atom to the i atom
c     - rr2(i,j)      : squared relative distance of j-th to i-th atom
c     - frepx/y/z(i)  : repulsive force for the i-th atom
c                       (reduced units : eV/A= 1.602177e-4 dyne)
c     - fhfx/y/z(i)   : Hellmann-Feynman force for the i-th atom
c                       (reduced units : eV/A= 1.602177e-4 dyne)
c     - ftotx/y/z(i)  : total force for the i-th atom
c                       (reduced units : eV/A= 1.602177e-4 dyne)
c
c***************************************************************************
c
c     Include files
c
      include 'param.inc'
      include 'pos.inc'
      include 'vel.inc'
      include 'acc.inc'
      include 'dist.inc'
      include 'latt.inc'
      include 'tb.inc'
      include 'frep.inc'
      include 'fhf.inc'
      include 'ftot.inc'
      include 'erg.inc'
      include 'eigen.inc'
      include 'func.inc'
      include 'lst.inc'
      include 'simul.inc'
      include 'start.inc'
c
      real*8 ax(n), ay(n), az(n)
c
c     Evolution of the system according to 'VELOCITY VERLET' algorithm
c
      do 1 i=1,n
        x(i)=(x(i)+ dt*vx(i) + dtsqr*axold(i))
        y(i)=(y(i)+ dt*vy(i) + dtsqr*ayold(i))
        z(i)=(z(i)+ dt*vz(i) + dtsqr*azold(i))
c
        x(i)=x(i)-cell*(dint(x(i)/cell+1.d0)-1.d0)
        y(i)=y(i)-cell*(dint(y(i)/cell+1.d0)-1.d0)
        z(i)=z(i)-cell*(dint(z(i)/cell+1.d0)-1.d0)
1     continue
c
      do 2 i=1,n
        do 3 j=1,n
          xx(j,i)= 0.d0
          yy(j,i)= 0.d0
          zz(j,i)= 0.d0
          rr2(j,i)= 0.d0
3       continue
2     continue
c
c     Calculation of the relative position and of the relative
c     squared distance of any atom to the nearest image of any other
c     one, according to periodic boundary conditions.
c
      do 4 i=1,n
       do 5 j=i+1,n
        xx(i,j)= x(j)-x(i)
        xx(i,j)=xx(i,j)-cell*(dint((xx(i,j)/cell2+3.d0)/2.d0)-1.d0)
        yy(i,j)= y(j)-y(i)
        yy(i,j)=yy(i,j)-cell*(dint((yy(i,j)/cell2+3.d0)/2.d0)-1.d0)
        zz(i,j)= z(j)-z(i)
        zz(i,j)=zz(i,j)-cell*(dint((zz(i,j)/cell2+3.d0)/2.d0)-1.d0)
        rr2(i,j)= xx(i,j)**2 + yy(i,j)**2 + zz(i,j)**2
c
        xx(j,i)=-xx(i,j)
        yy(j,i)=-yy(i,j)
        zz(j,i)=-zz(i,j)
        rr2(j,i)= rr2(i,j)
5      continue
4     continue
c
c     Compute Verlet list
c
      do 10 i=1,n
        ilst=0
        jlst=0
        do 20 j=1,n
          if ( i .eq. j ) goto 20
          if ( rr2(i,j) .gt. cutoff ) goto 20
          ilst=ilst+1
          list(i,ilst)=j
          if ( i .gt. j ) goto 20
          jlst=jlst+1
          list1(i,jlst)=j
20      continue
        nlst(i)=ilst
        nlst1(i)=jlst
10    continue          
c
c     Compute repulsive forces 
c
      call repuls(itype)
c
c     Compute and dizgonalizes tight-binding hamiltonian matrix.
c    
      call htb
c
c     Compute of Hellmann-feynman forces 
c
      call feynman
c
      do 8 i=1,n
c
c     Compute total forces and accelerations
c 
        ftotx(i)= frepx(i) + fhfx(i)
        ax(i)= ftotx(i)/mass
c
        ftoty(i)= frepy(i) + fhfy(i)
        ay(i)= ftoty(i)/mass
c
        ftotz(i)= frepz(i) + fhfz(i)
        az(i)= ftotz(i)/mass
c
c       Update velocities 
c                    
        vx(i)= vx(i) + dthalf * (ax(i) + axold(i))
        vy(i)= vy(i) + dthalf * (ay(i) + ayold(i))
        vz(i)= vz(i) + dthalf * (az(i) + azold(i))
c
c       Store accelerations
c
        axold(i)= ax(i)
        ayold(i)= ay(i)
        azold(i)= az(i)
c
8     continue
c
c     Compute istantaneous temperature
c    
      call temper
c
      return
      end         
c****************************************************************************
      subroutine meandisp(istep,ndisp0)
c**************************************************************************** 
c     RELEASE TBMD.2.0
c     Last revision: November 21, 1996 
c****************************************************************************
c     Contact author:    Luciano Colombo
c                        e-mail:   colombo@mi.infn.it
c****************************************************************************
c
c     Subroutine MEANDISP computes mean square displacement at every step.
c
c****************************************************************************
c
c     Include files
c 
      include 'param.inc'
      include 'pos.inc'
      include 'vel.inc'
      include 'acc.inc'
      include 'simul.inc'      
c
      real*8 rx0(n), ry0(n), rz0(n)
      real*8 rx(n), ry(n), rz(n)
      real*8 dsum
c
      if ( istep .eq. ndisp0 ) then
c
        do 10 i=1,n
          rx0(i)=x(i)
          ry0(i)=y(i)
          rz0(i)=z(i)
          rx(i)=rx0(i)
          ry(i)=ry0(i)
          rz(i)=rz0(i)
10      continue
c
        disp=0.d0
c
        write(83,*) istep,disp
c
      else
c
        dsum= 0.d0
c
        do 20 i=1,n
c
          rx(i)= (rx(i)+dt*vx(i)+dtsqr*axold(i))
          ry(i)= (ry(i)+dt*vy(i)+dtsqr*ayold(i))
          rz(i)= (rz(i)+dt*vz(i)+dtsqr*azold(i))
c
          dsum= dsum + (rx(i)-rx0(i))**2 + (ry(i)-ry0(i))**2 +
     *                 (rz(i)-rz0(i))**2
c
20      continue
c
        disp= dsum/dfloat(n)
c
        write(83,*) istep,disp
c    
      endif
c
      return
      end
c****************************************************************************
      subroutine nearangle(istep,nbond0,nbondf)
c**************************************************************************** 
c     RELEASE TBMD.2.0
c     Last revision: November 21, 1996 
c****************************************************************************
c     Contact author:    Luciano Colombo
c                        e-mail:   colombo@mi.infn.it
c****************************************************************************
c
c     Subroutine NEARANGLE computes bond angle distribution and
c     atomic coordination number. 
c
c      Variables
c
c     - xx/yy/zzm(nnear) : vectors of nearest neighbours position
c     - rr2m(nnear)      : nearest neighbours distances
c     - rmax             : maximum distance of a nearest neighbour
c     - avangl(nsudd)    : average bond angle distribution
c     - avprim(nnear)    : average atomic coordination number 
c     - near             : n. of nearest neighbours to every atom
c     - nnear            : maximum number of neighbours (if nnear is exceeded
c                          the computation of avangl and avprim could fail).
c
c     ******************************************************************
c
c     Include files
c
      include 'param.inc'
c
      parameter (nsuddb= 300, nnear=20)
      include 'simul.inc'
      include 'dist.inc'
      include 'latt.inc'
c
      real*8 xxm(nnear), yym(nnear), zzm(nnear), rr2m(nnear)
      real*8 rr2d
      real*8 angcos, angpos, angsud, anglen
      real*8 avangl(nsuddb)
      integer near
      real*8 avprim(nnear)
      real*8 pgreek
c
      data avangl/nsuddb*0.d0/
      data avprim/nnear*0.d0/
c
      pgreek= dacos(-1.d0)/180.d0
c
c     Widht of intervals
c
      angsud= 180.d0/dfloat(nsuddb)
c
c     Compute atomic coordination number 
c 
      do 10 i=1,n
        near=0 
        do 20 j=1,n
          if ( j .eq. i ) goto 20
          if ( rr2(i,j) .gt. bcutof ) goto 20
          near= near+1
          xxm(near)= xx(i,j)
          yym(near)= yy(i,j)
          zzm(near)= zz(i,j)
          rr2m(near)= rr2(i,j)
20      continue
c
        avprim(near)= avprim(near)+1.d0
c
c     Compute bond angle distribution
c
        do 30 ik=1,near
          do 40 jk=ik+1,near
            rr2d= (xxm(ik)-xxm(jk))**2+
     1            (yym(ik)-yym(jk))**2+
     2            (zzm(ik)-zzm(jk))**2
            angcos= (rr2m(jk)+rr2m(ik)-rr2d)/
     *              (2.d0*dsqrt(rr2m(ik))*dsqrt(rr2m(jk)))
            anglen= dacos(angcos)/pgreek
            npos= int(anglen/angsud)+1
            avangl(npos)= avangl(npos)+1.d0
40        continue
30      continue
c
10    continue
c
c     Output 
c
      if ( istep .eq. nbondf ) then
        do 70 isudd=1,nsuddb
          angpos= angsud*(dfloat(isudd)-0.5d0)
          avangl(isudd)= avangl(isudd)/dfloat(nbondf-nbond0+1)
     *                      /dfloat(n)
          write(91,*) angpos,avangl(isudd)
70      continue
c
        do 80 ik=1,nnear
          avprim(ik)= avprim(ik)/dfloat(nbondf-nbond0+1)
     *                  /dfloat(n)
          write(88,*) ik,avprim(ik)
80      continue
      endif
c
      return
      end
c****************************************************************************
      subroutine repuls(itype)
c**************************************************************************** 
c     RELEASE TBMD.2.0
c     Last revision: November 21, 1996 
c****************************************************************************
c     Contact author:    L. Colombo 
c                        e-mail:  colombo@mi.infn.it 
c****************************************************************************
c
c     Subroutine repuls computes repulsive forces and energy
c
c****************************************************************************
c
c     Include files
c
      include 'param.inc'
      include 'dist.inc'
      include 'frep.inc'
      include 'potz.inc'
      include 'latt.inc'
      include 'lst.inc'
      include 'erg.inc'
c
      real*8 usum, phi, dphi 
      real*8 ftm1, f1sx, f1sy, f1sz, ftm2 
      real*8 rrn, xxn, yyn, zzn
      real*8 eri(n)
c
      usum= 0.d0
c 
c The repulsive potential is slightly different for carbon and silicon.
c Carbon:  fact1=phi0 
c          fact2=d0
c          (the power law, as well as the exponential tail depend on d0) 
c Silicon: fact1=1
c          (phi0 is not defined) 
c          fact2=r0
c          (the power law, as well as the exponential tail depend on r0)
c 
           if (itype .eq. 1) then 
              fact1 = phi0 
              fact2 = d0    
           end if 
           if (itype .eq. 2) then 
              fact1 = 1.d0 
              fact2 = r0 
           end if 
c
c Compute: sum ( phi (rij) ) for each i  
c
      do 5 i= 1, n
       eri(i)= 0.d0
       frepx(i)= 0.d0
       frepy(i)= 0.d0
       frepz(i)= 0.d0
 5    continue
c
      do 1 i= 1, n
         do 2 ji= 1, nlst1(i)
            j= list1(i,ji)
            rrn= dsqrt(rr2(i,j))
            phi=fact1*(fact2/rrn)**em *
     >          dexp( em* ( -( rrn/dc )**emc + (fact2/dc )**emc ) )
            eri(i) = eri(i) + phi
            eri(j) = eri(j) + phi
 2    continue
 1    continue
c
c Compute: total repulsive energy, and repulsive force 
c
      do 3 i=1, n
         usum = usum + c0 + 
     1          c1 * eri(i) + c2 * eri(i) ** 2 +
     2          c3 * eri(i) ** 3 + c4 * eri(i) ** 4
         ftm1= c1 + 2.d0 * c2 * eri(i) + 
     1         3.d0 * c3 * eri(i) **2 + 4.d0 * c4 * eri(i) **3
         f1sx= 0.d0
         f1sy= 0.d0
         f1sz= 0.d0 
c
         do 4 ji= 1, nlst(i)
            j= list(i,ji)
            rrn= dsqrt(rr2(i,j))
            phi=fact1*( fact2/rrn )**em *
     >          dexp( em* ( -( rrn/dc )**emc + (fact2/dc )**emc ) )
            dphi= phi * ( 1.d0 + emc * ( rrn/dc ) ** emc)*em/rrn
            xxn= xx(j,i) / rrn
            yyn= yy(j,i) / rrn
            zzn= zz(j,i) / rrn 
         ftm2= c1 + 2.d0 * c2 * eri(j) + 
     1         3.d0 * c3 * eri(j) **2 + 4.d0 * c4 * eri(j) **3
c         
            f1sx= f1sx + dphi * xxn
            f1sy= f1sy + dphi * yyn 
            f1sz= f1sz + dphi * zzn 
            frepx(i)= frepx(i) + ftm2 * dphi * xxn 
            frepy(i)= frepy(i) + ftm2 * dphi * yyn 
            frepz(i)= frepz(i) + ftm2 * dphi * zzn 
 4        continue
          frepx(i) = frepx(i) + ftm1 * f1sx
          frepy(i) = frepy(i) + ftm1 * f1sy
          frepz(i) = frepz(i) + ftm1 * f1sz 
 3    continue
c
      utot= usum/dfloat(n)  
c      write(*,*) 'utot=',utot
c
      return
      end
c****************************************************************************
      subroutine temper
c****************************************************************************
c     RELEASE TBMD.2.0
c     Last revision: November 21, 1996 
c****************************************************************************
c     Contact author:    Luciano Colombo
c                        e-mail:   colombo@mi.infn.it
c****************************************************************************
c
c
c     Subroutine TEMPER computes the temperature of the system from
c     the total kinetic energy, using equipartition theorem.
c
c     total kinetic energy = 1/2 * kboltz*temp * ( degrees of freedom )
c
c     The degrees of freedom of the periodic system containing n atoms are
c     3*n-3 
c
c     Variables  
c
c     - vx/y/z(i)     : velocity of the i-th atom
c     - mass          : atomic mass
c     - kin(i)        : kinetic energy of the i-th atom
c     - kinsum        : total kinetic energy of the system
c     - kintot        : total kinetic energy per atom
c     - kboltz        : Boltzmann's constant
c     - temp          : temperature of the system
c
c****************************************************************************
c
c     Include files
c
      include 'param.inc'
      include 'vel.inc'
      include 'latt.inc'
      include 'func.inc'
      include 'erg.inc'
c
      real*8 kinsum
c
c     Kinetic energy of atoms
c
      kinsum= 0.d0
c
      do 100 j=1,n
         kinsum= kinsum+vx(j)**2+vy(j)**2+vz(j)**2
100   continue
c
      kintot=0.5d0*mass*kinsum/dfloat(n)
c
c     Compute temperature from equipartition theorem
c
      tempi=mass*kinsum/(3.d0*dfloat(n)-3.d0)/kboltz
c
      return
      end
c****************************************************************************
      subroutine bondlength(istep,nlength0,nlengthf,bcutof)
c****************************************************************************
c     RELEASE TBMD.2.0
c     Last revision: November 21, 1996 
c****************************************************************************
c     Contact author:    Luciano Colombo
c                        e-mail:   colombo@mi.infn.it
c****************************************************************************
c
c     Subroutine BONDLENGTH  computes the bond-lenght distribution
c     
c
c     Variables
c
c     - nlsuddc         : number of different intervals for computing  
c                         bondlength distribution
c     - xx/yy/zzm       : vector of relative distance between atoms
c     - rrm             : relative distance between atoms
c     - length(nlsuddc) : bondlength distribution function
c
c****************************************************************************
c
c     Include files and parameters
c
      include 'param.inc'
      parameter ( nlsuddc= 300 )
      include 'dist.inc'
      include 'latt.inc'
      include 'pos.inc'
c
      real*8 xxm, yym, zzm, rrm
      real*8 xxn, yyn, zzn
      real*8 rmin, rmax, rsudd, rpos
      real*8 length(nlsuddc)
      real*8 bcutof 
c
      data length/nlsuddc*0.d0/
      bcut=dsqrt(bcutof)
c
c     Range for bondlength distribution
c
      rmax= bcut 
      nvic=0
      rmin= 0.d0 
c
c     Widht of intervals 
c 
      rsudd= ( rmax - rmin )/ dfloat(nlsuddc)
c
c     Calculation of length distribution function
c
      do 10 i=1,n
        do 20 j=i+1,n
          xxn= xx(i,j)
          yyn= yy(i,j)
          zzn= zz(i,j)
          do 30 i1=0,1
            xxm= xxn-dfloat(i1)*(xxn/dabs(xxn))*cell
            do 40 i2=0,1
              yym= yyn-dfloat(i2)*(yyn/dabs(yyn))*cell
              do 50 i3=0,1
                zzm= zzn-dfloat(i3)*(zzn/dabs(zzn))*cell
                rrm=dsqrt (xxm**2+yym**2+zzm**2)
                if ( rrm .ge. rmax ) goto 50
                nvic=nvic+1
                npos= int((rrm-rmin)/rsudd)+1
                length(npos)= length(npos)+1
50            continue
40          continue
30        continue
20      continue
10    continue
c
c     Output 
c
      if ( istep .eq. nlengthf ) then
        do 70 isudd=1,nlsuddc
          rpos= rmin+rsudd*(dfloat(isudd)-0.5d0)
          length(isudd)= length(isudd)/dfloat(nlengthf-nlength0+1)
     >       /dfloat(nvic)
          write(34,*) rpos,length(isudd)
70      continue
      endif
c
      return  
      end  
c***************** RANDOM NUMBER GENERATOR ***********************
c
      FUNCTION RAN1(IDUM)
      DIMENSION R(197)
      PARAMETER (M1=259200,IA1=7141,IC1=54773,RM1=3.8580247E-6)
      PARAMETER (M2=134456,IA2=8121,IC2=28411,RM2=7.4373773E-6)
      PARAMETER (M3=243000,IA3=4561,IC3=51349)
      DATA IFF /0/
      IF (IDUM.LT.0.OR.IFF.EQ.0) THEN
        IFF=1
        IX1=MOD(IC1-IDUM,M1)
        IX1=MOD(IA1*IX1+IC1,M1)
        IX2=MOD(IX1,M2)
        IX1=MOD(IA1*IX1+IC1,M1)
        IX3=MOD(IX1,M3)
        DO 11 J=1,197
          IX1=MOD(IA1*IX1+IC1,M1)
          IX2=MOD(IA2*IX2+IC2,M2)
          R(J)=(FLOAT(IX1)+FLOAT(IX2)*RM2)*RM1
11      CONTINUE
        IDUM=1
      ENDIF
      IX1=MOD(IA1*IX1+IC1,M1)
      IX2=MOD(IA2*IX2+IC2,M2)
      IX3=MOD(IA3*IX3+IC3,M3)
      J=1+(97*IX3)/M3
      IF(J.GT.197.OR.J.LT.1)PAUSE
      RAN1=R(J)
      R(J)=(FLOAT(IX1)+FLOAT(IX2)*RM2)*RM1
      RETURN
      END
c****************************************************************************
